const axios = require('axios');
const { HttpsProxyAgent } = require('https-proxy-agent');
const { SocksProxyAgent } = require('socks-proxy-agent');
const { HttpProxyAgent } = require('http-proxy-agent');
const logger = require('../utils/logger');
const { TestResult } = require('../models');
const net = require('net');

class SpeedTester {
  constructor() {
    this.testUrls = process.env.SPEED_TEST_URLS?.split(',') || [
      'https://www.google.com',
      'https://www.youtube.com',
      'https://www.github.com',
      'https://httpbin.org/get'
    ];
    this.timeout = parseInt(process.env.SPEED_TEST_TIMEOUT) || 10000;
  }

  /**
   * TCP端口连通性检测
   */
  async testTcpConnectivity(server, port, timeout = 5000) {
    return new Promise((resolve) => {
      const socket = new net.Socket();
      let isConnected = false;

      socket.setTimeout(timeout);

      socket.on('connect', () => {
        isConnected = true;
        socket.destroy();
        resolve({ success: true });
      });

      socket.on('timeout', () => {
        socket.destroy();
        if (!isConnected) resolve({ success: false, error: '连接超时' });
      });

      socket.on('error', (err) => {
        socket.destroy();
        if (!isConnected) resolve({ success: false, error: err.message });
      });

      socket.connect(port, server);
    });
  }

  /**
   * 测试单个节点连通性（只做TCP端口探测）
   */
  async testNode(node) {
    const startTime = Date.now();
    let testResult = {
      nodeId: node.id,
      testTime: new Date(),
      isSuccess: false,
      latency: null,
      downloadSpeed: null, // 保留字段但不测试
      uploadSpeed: null,   // 保留字段但不测试
      packetLoss: null,
      testUrl: null,
      errorMessage: null,
      testDuration: null,
      ipAddress: null,
      location: null
    };

    try {
      logger.info(`开始测试节点连通性: ${node.name} (${node.type})`);

      // 验证节点配置的有效性
      const validationResult = this.validateNode(node);
      if (!validationResult.isValid) {
        throw new Error(validationResult.error);
      }

      // 直接检测 server:port 的 TCP 连通性
      const tcpResult = await this.testTcpConnectivity(node.server, node.port, this.timeout);
      testResult.isSuccess = tcpResult.success;
      testResult.errorMessage = tcpResult.error || null;

      testResult.testDuration = Date.now() - startTime;

      if (testResult.isSuccess) {
        logger.info(`✅ 节点TCP连通性测试成功: ${node.name}`);
      } else {
        logger.warn(`❌ 节点TCP连通性测试失败: ${node.name} - ${testResult.errorMessage || '连接超时'}`);
      }
    } catch (error) {
      testResult.errorMessage = error.message;
      testResult.testDuration = Date.now() - startTime;
      logger.error(`❌ 节点测试异常: ${node.name} - ${error.message}`);
    }

    // 保存测试结果
    await this.saveTestResult(testResult);

    return testResult;
  }

  /**
   * 验证节点配置的有效性
   */
  validateNode(node) {
    // 检查必需的字段
    if (!node.name) {
      return { isValid: false, error: '节点名称不能为空' };
    }
    
    if (!node.type) {
      return { isValid: false, error: '节点类型不能为空' };
    }
    
    if (!node.server) {
      return { isValid: false, error: '服务器地址不能为空' };
    }
    
    if (!node.port || node.port <= 0 || node.port > 65535) {
      return { isValid: false, error: '端口号无效' };
    }

    // 验证server字段是否为有效的域名或IP地址
    const server = node.server.trim();
    
    // 只检查明显无效的情况，而不是过于严格的验证
    // 检查是否包含中文字符（通常表示这是描述而不是有效地址）
    if (/[\u4e00-\u9fa5]/.test(server)) {
      return { isValid: false, error: `服务器地址包含中文字符，无效: ${server}` };
    }
    
    // 检查是否是URL格式（不应该作为server地址）
    if (server.startsWith('http://') || server.startsWith('https://')) {
      return { isValid: false, error: `服务器地址不能是URL格式: ${server}` };
    }
    
    // 检查是否包含明显的无效字符（如空格、特殊符号等）
    if (/[<>:"\\|?*]/.test(server)) {
      return { isValid: false, error: `服务器地址包含无效字符: ${server}` };
    }
    
    // 检查是否为空或只包含空白字符
    if (!server || server.trim() === '') {
      return { isValid: false, error: '服务器地址不能为空' };
    }

    // 对于SS/SSR节点，检查密码
    if ((node.type === 'ss' || node.type === 'ssr') && !node.password) {
      return { isValid: false, error: 'SS/SSR节点必须设置密码' };
    }

    // 对于Vmess节点，检查UUID
    if (node.type === 'vmess' && !node.uuid) {
      return { isValid: false, error: 'Vmess节点必须设置UUID' };
    }

    // 对于Trojan节点，检查密码
    if (node.type === 'trojan' && !node.password) {
      return { isValid: false, error: 'Trojan节点必须设置密码' };
    }

    return { isValid: true, error: null };
  }

  /**
   * 创建代理配置
   */
  createProxyConfig(node) {
    const baseConfig = {
      host: node.server,
      port: node.port,
      timeout: this.timeout
    };

    switch (node.type) {
      case 'ss':
        return this.createShadowsocksConfig(node, baseConfig);
      case 'ssr':
        return this.createShadowsocksRConfig(node, baseConfig);
      case 'vmess':
        return this.createVmessConfig(node, baseConfig);
      case 'trojan':
        return this.createTrojanConfig(node, baseConfig);
      case 'http':
      case 'https':
        return this.createHttpConfig(node, baseConfig);
      case 'socks5':
        return this.createSocks5Config(node, baseConfig);
      default:
        logger.warn(`不支持的代理类型: ${node.type} - ${node.name}`);
        return null;
    }
  }

  /**
   * 创建Shadowsocks代理配置
   */
  createShadowsocksConfig(node, baseConfig) {
    return {
      ...baseConfig,
      type: 'ss',
      password: node.password,
      method: node.cipher || node.method,
      auth: node.username && node.password ? {
        username: node.username,
        password: node.password
      } : null
    };
  }

  /**
   * 创建ShadowsocksR代理配置
   */
  createShadowsocksRConfig(node, baseConfig) {
    return {
      ...baseConfig,
      type: 'ssr',
      password: node.password,
      method: node.cipher || node.method,
      protocol: node.protocol,
      obfs: node.obfs,
      auth: node.username && node.password ? {
        username: node.username,
        password: node.password
      } : null
    };
  }

  /**
   * 创建Vmess代理配置
   */
  createVmessConfig(node, baseConfig) {
    return {
      ...baseConfig,
      type: 'vmess',
      uuid: node.uuid,
      alterId: node.alterId,
      network: node.network,
      tls: node.tls,
      sni: node.sni,
      wsPath: node.wsPath,
      wsHeaders: node.wsHeaders ? JSON.parse(node.wsHeaders) : {},
      auth: node.username && node.password ? {
        username: node.username,
        password: node.password
      } : null
    };
  }

  /**
   * 创建Trojan代理配置
   */
  createTrojanConfig(node, baseConfig) {
    return {
      ...baseConfig,
      type: 'trojan',
      password: node.password,
      tls: node.tls,
      sni: node.sni,
      auth: node.username && node.password ? {
        username: node.username,
        password: node.password
      } : null
    };
  }

  /**
   * 创建HTTP代理配置
   */
  createHttpConfig(node, baseConfig) {
    const config = {
      ...baseConfig,
      type: 'http'
    };

    if (node.username && node.password) {
      config.auth = {
        username: node.username,
        password: node.password
      };
    }

    if (node.tls) {
      config.https = true;
      config.sni = node.sni;
    }

    return config;
  }

  /**
   * 创建SOCKS5代理配置
   */
  createSocks5Config(node, baseConfig) {
    const config = {
      ...baseConfig,
      type: 'socks5'
    };

    if (node.username && node.password) {
      config.auth = {
        username: node.username,
        password: node.password
      };
    }

    return config;
  }

  /**
   * 测试基本连通性
   */
  async testBasicConnectivity(node, proxyConfig) {
    const startTime = Date.now();
    
    // 如果是直连测试
    if (node.type === 'direct') {
      return this.testDirectConnectivity();
    }
    
    // 对于高级代理类型，我们仍然尝试通过本地代理测试
    // 如果本地代理不可用，会在makeRequest中失败，这是正常的
    
    // 尝试多个测试URL
    for (const url of this.testUrls) {
      try {
        logger.debug(`尝试连接: ${node.name} -> ${url}`);
        
        const response = await this.makeRequest(node, proxyConfig, {
          method: 'HEAD',
          url: url,
          timeout: this.timeout,
          maxRedirects: 3
        });

        const latency = Date.now() - startTime;
        
        logger.debug(`连接成功: ${node.name} -> ${url} (${latency}ms)`);
        
        return {
          success: true,
          latency: latency,
          url: url,
          ipAddress: this.extractIpAddress(response),
          location: this.extractLocation(response)
        };
      } catch (error) {
        logger.debug(`连接失败: ${node.name} -> ${url} - ${error.message}`);
        continue;
      }
    }

    return {
      success: false,
      latency: null,
      url: null,
      ipAddress: null,
      location: null
    };
  }



  /**
   * 测试直连连通性
   */
  async testDirectConnectivity() {
    const startTime = Date.now();
    
    for (const url of this.testUrls) {
      try {
        logger.debug(`尝试直连: ${url}`);
        
        const response = await axios.get(url, {
          timeout: this.timeout,
          headers: {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'
          }
        });

        const latency = Date.now() - startTime;
        
        logger.debug(`直连成功: ${url} (${latency}ms)`);
        
        return {
          success: true,
          latency: latency,
          url: url,
          ipAddress: this.extractIpAddress(response),
          location: this.extractLocation(response)
        };
      } catch (error) {
        logger.debug(`直连失败: ${url} - ${error.message}`);
        continue;
      }
    }

    return {
      success: false,
      latency: null,
      url: null,
      ipAddress: null,
      location: null
    };
  }

  /**
   * 发送HTTP请求
   */
  async makeRequest(node, proxyConfig, requestConfig) {
    const agent = this.createProxyAgent(node, proxyConfig);
    
    if (!agent) {
      throw new Error(`无法创建代理Agent: ${proxyConfig.type}`);
    }
    
    logger.debug(`使用代理: ${node.name} - ${proxyConfig.type}://${proxyConfig.host}:${proxyConfig.port}`);
    
    const config = {
      ...requestConfig,
      httpsAgent: agent,
      httpAgent: agent,
      headers: {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36',
        ...requestConfig.headers
      }
    };

    return axios(config);
  }

  /**
   * 创建代理Agent
   */
  createProxyAgent(node, proxyConfig) {
    try {
      logger.debug(`创建代理Agent: ${node.name} - ${proxyConfig.type}`);
      
      switch (proxyConfig.type) {
        case 'http':
        case 'https':
          // HTTP/HTTPS代理
          let httpProxyUrl = `${proxyConfig.type}://${proxyConfig.host}:${proxyConfig.port}`;
          
          // 如果有认证信息，添加到URL中
          if (proxyConfig.auth) {
            const { username, password } = proxyConfig.auth;
            httpProxyUrl = `${proxyConfig.type}://${username}:${password}@${proxyConfig.host}:${proxyConfig.port}`;
          }
          
          const httpAgent = new HttpsProxyAgent(httpProxyUrl);
          logger.debug(`HTTP代理创建成功: ${node.name} - ${httpProxyUrl}`);
          return httpAgent;
          
        case 'socks5':
          // SOCKS5代理
          let socksProxyUrl = `socks5://${proxyConfig.host}:${proxyConfig.port}`;
          
          // 如果有认证信息，添加到URL中
          if (proxyConfig.auth) {
            const { username, password } = proxyConfig.auth;
            socksProxyUrl = `socks5://${username}:${password}@${proxyConfig.host}:${proxyConfig.port}`;
          }
          
          const socksAgent = new SocksProxyAgent(socksProxyUrl);
          logger.debug(`SOCKS5代理创建成功: ${node.name} - ${socksProxyUrl}`);
          return socksAgent;
          
        case 'ss':
        case 'ssr':
        case 'vmess':
        case 'trojan':
          // 对于这些高级代理类型，我们需要通过本地Clash代理来测试
          // 因为Node.js没有直接的vmess/ss客户端库
          logger.info(`通过本地Clash代理测试: ${proxyConfig.type} - ${node.name}`);
          
          // 使用本地Clash混合端口
          const localProxyUrl = 'http://127.0.0.1:7890';
          const localAgent = new HttpsProxyAgent(localProxyUrl);
          logger.debug(`使用本地Clash代理: ${node.name} - ${localProxyUrl}`);
          return localAgent;
          
        default:
          logger.warn(`不支持的代理类型: ${proxyConfig.type} - ${node.name}`);
          return null;
      }
      
    } catch (error) {
      logger.error(`创建代理Agent失败: ${node.name}`, { error: error.message, type: proxyConfig.type });
      return null;
    }
  }

  /**
   * 提取IP地址
   */
  extractIpAddress(response) {
    // 从响应头中提取IP地址
    return response.headers['x-forwarded-for'] || 
           response.headers['x-real-ip'] || 
           response.headers['cf-connecting-ip'] ||
           'unknown';
  }

  /**
   * 提取位置信息
   */
  extractLocation(response) {
    // 从响应头中提取位置信息
    if (response.headers['cf-ray']) {
      return 'Cloudflare';
    }
    if (response.headers['cf-ipcountry']) {
      return response.headers['cf-ipcountry'];
    }
    return 'unknown';
  }

  /**
   * 保存测试结果
   */
  async saveTestResult(testResult) {
    try {
      await TestResult.create(testResult);
      logger.debug(`测试结果已保存: ${testResult.nodeId}`);
    } catch (error) {
      logger.error(`保存测试结果失败: ${error.message}`);
    }
  }

  /**
   * 批量测试节点
   */
  async testNodes(nodes) {
    logger.info(`开始批量测试 ${nodes.length} 个节点`);
    
    const results = [];
    const concurrency = parseInt(process.env.CONCURRENCY) || 5;
    
    // 分批处理，控制并发数
    for (let i = 0; i < nodes.length; i += concurrency) {
      const batch = nodes.slice(i, i + concurrency);
      const batchPromises = batch.map(node => this.testNode(node));
      
      const batchResults = await Promise.allSettled(batchPromises);
      
      batchResults.forEach((result, index) => {
        if (result.status === 'fulfilled') {
          results.push(result.value);
        } else {
          logger.error(`节点测试失败: ${batch[index].name} - ${result.reason}`);
        }
      });
      
      // 批次间延迟，避免过于频繁的请求
      if (i + concurrency < nodes.length) {
        await new Promise(resolve => setTimeout(resolve, 1000));
      }
    }
    
    const successCount = results.filter(r => r.isSuccess).length;
    logger.info(`批量测试完成: ${successCount}/${nodes.length} 个节点连通`);
    
    return results;
  }

  /**
   * 数组分块
   */
  chunkArray(array, size) {
    const chunks = [];
    for (let i = 0; i < array.length; i += size) {
      chunks.push(array.slice(i, i + size));
    }
    return chunks;
  }
}

module.exports = SpeedTester;
